#include <avr/io.h>

#define StartTimer()				(TCCR3B |=0x01, ETIMSK |=0x04)	

#define ResetTimer()				(TCNT3H = 0xE0, TCNT3L = 0xE8)

#define SetPorts()	 (DDRA = 0x00, DDRB = 0x90, DDRC = 0xFF, 	\
					  DDRD = 0xAE, DDRE = 0x03, DDRF = 0x00, 	\
					  DDRG = 0x07,								\
					  PORTA = 0x00, PORTB = 0x90, PORTC = 0xFF, \
					  PORTD = 0xA4, PORTE = 0x00, PORTF = 0x00, \
					  PORTG = 0x07) 

#define StartPwm0() (TCCR0 |=(1<<CS00), TCCR0 |=(1<<WGM00),\
					  TCCR0 |=(1<<COM01),TCCR0 |=(1<<COM00) )

#define StartPwm2() (TCCR2 |=(1<<CS00), TCCR2 |=(1<<WGM00),\
					  TCCR2 |=(1<<COM01),TCCR2 |=(1<<COM00) ) 

#define StopPwm0()	(TCCR0 = 0)
#define StopPwm2()	(TCCR2 = 0)


#define LockSpeedSet(a)				(OCR2 = a)
#define LockSpeedRead()				(OCR2)
#define LockSpeedIncr()				(OCR2++)
#define LockSpeedDecr()				(OCR2--)  
					  
#define SetMotorCounter()			(TCCR1B = 0xC7)	 



#define EnableAdConverter()			(ADCSRA |= (1<<ADEN))
#define SetAdVoltageReference()		(ADMUX |= (1<<REFS0))
#define EnableAdInterrupt()			(ADCSRA |= (1<<ADIE))
#define SetAdDivFactor()			(ADCSRA |= 7)
#define StartAdConvert()			(ADCSRA |= (1<<ADSC))

#define ResetAdMux()				(ADMUX &=~ 7)
#define ADmuxGardenLight()			(ADMUX |=1)
#define ADmuxFotoTemp()				(ADMUX |=3)
#define ADmuxLowVoltage()			(ADMUX |=0)
#define ADmuxHighTemp()				(ADMUX |=2)

#define ReadAdValue(a)			(a=0, a|=ADCL, a|=(ADCH<<8))


//inputs
#define SwFontana()					(((PINB&0x40)==0) ? 0 : 1 )
#define	SwSaht()					(((PING&0x10)==0) ? 0 : 1 )
#define SwWathering()				(((PINB&0x20)==0) ? 0 : 1 ) //PB5
#define SwRaining()					(((PIND&0x01)==0) ? 0 : 1 ) //PD0
#define SwGardenLight()				(((PINB&0x04)==0) ? 0 : 1 ) //PB2
#define SwOutsideLight()			(((PINB&0x08)==0) ? 0 : 1 ) //PB3

#define OutLedRunOn()				(PORTE |= 0x01)				//PE0
#define OutLedRunOff()				(PORTE &= ~0x01)			//PE0
#define OutLedRunToggle()			(PORTE ^= 0x01)				//PE0

#define OutLedErrorOn()				(PORTE |= 0x02)				//PE1
#define OutLedErrorOff()			(PORTE &= ~0x02)			//PE1

#define OutFontanaToggle()			(PORTG ^= 0x01)				//PG0
#define OutFontanaOn()				(PORTG &= ~0x01)			//PG0
#define OutFontanaOff()				(PORTG |= 0x01)				//PG0
#define OutFontanaState()			(((PORTG & 0x01)==0) ? 1 : 0)

#define OutWatheringToggle()		(PORTC ^= 0x04)				//PC2
#define OutWatheringOn()			(PORTC &= ~0x04)			//PC2
#define OutWatheringOff()			(PORTC |= 0x04)				//PC2
#define OutWatheringState()			(((PORTC & 0x04)==0) ? 1 : 0)


#define OutSahtToggle()				(PORTC ^= 0x02)				//PC1
#define OutSahtOn()					(PORTC &= ~0x02)			//PC1
#define OutSahtOff()				(PORTC |= 0x02)				//PC1


#define OutGardenLightToggle()		(PORTD ^= 0x04)				//PD2
#define OutGardenLightOn()			(PORTD &= ~0x04)			//PD2
#define	OutGardenLightOff()			(PORTD |= 0x04)				//PD2
#define OutGardenLightState()		(((PORTD & 0x04)==0) ? 1 : 0)

#define OutOutsideLightToggle()		(PORTG ^= 0x02)				//PG1
#define OutOutsideLightOn()			(PORTG &= ~0x02)			//PG1
#define OutOutsideLightOff()		(PORTG |= 0x02)				//PG1
#define OutOutsideLightState()		(((PORTG & 0x02)==0) ? 1 : 0)

#define OutStToggle()				(PORTD ^= 0x02)				//PD1
#define OutStOn()					(PORTD &= ~0x02)			//PD1
#define OutStOff()					(PORTD |= 0x02)				//PD1
#define OutStState()				(((PORTD & 0x02)==0) ? 1 : 0)

#define OutBeepToggle()				(PORTD ^= 0x80)				//PD7
#define OutBeepOn()					(PORTD &= ~0x80)			//PD7
#define OutBeepOff()				(PORTD |= 0x80)				//PD7
#define OutBeepState()				(((PORTD & 0x80)==0) ? 1 : 0)

#define OutLightFlashToggle()		(PORTC ^= 0x01)				//PC0
#define OutLightFlashOn()			(PORTC &= ~0x01)			//PC0
#define OutLightFlashOff()			(PORTC |= 0x01)				//PC0
#define OutLightFlashState()		(((PORTC & 0x01)==0) ? 1 : 0)

#define OutLockPwmToggle()			(PORTD ^= 0x08)				//PD3
#define OutLockPwmOn()				(PORTD &= ~0x08)			//PD3
#define OutLockPwmOff()				(PORTD |= 0x08)				//PD3
#define OutLockPwmState()			(((PORTD & 0x08)==0) ? 1 : 0)

//__________________________________________________________________________


//__________________________________________________________________________
#define After22()					(((PING&0x08)==0) ? 0 : 1 ) //PG3

#define SwitchBlock()				(((PINE&0x04)==0) ? 0 : 1 ) //PE2

#define SwEndClose()				(((PINA&0x10)==0) ? 1 : 0 ) //PA4
#define SwEndOpen()					(((PINA&0x40)==0) ? 1 : 0 ) //PA6
#define SwFoto()					(((PINA&0x04)==0) ? 1 : 0 ) //PA2
#define SwUserClose()				(((PINA&0x02)==0) ? 0 : 1 ) //PA1
#define SwUserOpen()				(((PINA&0x01)==0) ? 0 : 1 ) //PA0

#define SwOverCurrent4A()			(((PINE&0x08)==0) ? 1 : 0 ) //PE3
#define SwOverCurrent8A()			(((PINA&0x80)==0) ? 1 : 0 ) //PA7

//__________________________________________________________________________


#define OutFotoToggle()				(PORTD ^= 0x20)				//PD5
#define OutFotoOn()					(PORTD &= ~0x20)			//PD5
#define OutFotoOff()				(PORTD |= 0x20)				//PD5
#define OutFotoState()				(((PORTD & 0x20)==0) ? 1 : 0)

//__________________________________________________________________________
//Big door motor drivers

#define OutDoorClose()				(PORTC &= ~0x40, PORTC |= 0x08)
#define OutDoorOpen()				(PORTC &= ~0x08, PORTC |= 0x40 )
#define OutDoorStop()				(PORTC |= 0x48 )

#define BlockPwm1()					(DDRB &= ~0x10)
#define UnblockPwm1()				(DDRB |= 0x10)

#define DoorSpeedSet(a)				(OCR0 = a)
#define DoorSpeedRead()				(OCR0)
#define DoorSpeedIncr()				(OCR0++)
#define DoorSpeedDecr()				(OCR0--)
#define DoorCounterReset(a)			(TCNT1 = a)
#define DoorCounterRead()			(TCNT1)
//__________________________________________________________________________
//Big door lock drivers

#define SwDoorLocked()				(((PINA&0x20)==0) ? 1 : 0 ) //PA5
#define SwDoorUnlocked()			(((PINA&0x08)==0) ? 1 : 0 ) //PA3
#define OutLockDoor()				(PORTC |= 0x80, PORTC &= ~0x10 )
#define OutUnLockDoor()				(PORTC |= 0x10, PORTC &= ~0x80 )
#define OutLockOff()				(PORTC |= 0x10, PORTC |= 0x80 )

//__________________________________________________________________________
//Small door lock drivers
#define SwUserLock1()				(((PINE&0x10)==0) ? 0 : 1 ) //PE4
#define SwUserUnlock1()				(((PINE&0x20)==0) ? 0 : 1 ) //PE5
#define SwDoorLocked1()				(((PINB&0x02)==0) ? 1 : 0 ) //PB1
#define SwDoorUnlocked1()			(((PINE&0x40)==0) ? 1 : 0 ) //PE6
#define SwDoor1Closed()				(((PINE&0x80)==0) ? 1 : 0 ) //PE7
#define SwOverCurrent28()			(((PINB&0x01)==0) ? 1 : 0 ) //PB0
#define OutLockDoor1()				(PORTG |= 0x04, PORTC &= ~0x20 )
#define OutUnLockDoor1()			(PORTC |= 0x20, PORTG &= ~0x04 )
#define OutLockOff1()				(PORTC |= 0x20, PORTG |= 0x04 )
