#include "app.h"
#include "config.h"
#include "board_drv.h"
#include "haus_control.h"
#include "app_lib.h"
#include "avr\interrupt.h"

unsigned char CloseDoor(unsigned char speed){

	//if door is not unlocked
	if( !SwDoorUnlocked()){
		
		//funktion is not executed
		return (0);
	}
	
	//if door is not closed and if door standing
	if(!door.state && !SwEndClose() && !SwFoto() ){
		
		if(door.restarted){
			
			//Set target speed for motor
			door.target_speed = speed;
		}
		else{
			//Set target speed for motor
			door.target_speed= DOOR_RESET_SPEED;
		}
			
		//Reset timer for target speed
		t_door_target_speed = 0;

		//Turn on motor to close door
		OutDoorClose();
		
		//Set begin speed for motor
		DoorSpeedSet( DOOR_START_SPEED );
			
		//now motor running
		door.state = CLOSE;		
	}
	else if(SwEndClose() && !door.state){
		
		return (1);
	}	 
			
	//Run Application Service
	Service();	

	//command is not executed
	return 0;
}

unsigned char OpenDoor(unsigned char speed){
	
	//if door is not unlocked
	if( !SwDoorUnlocked() ){
		
		//funktion is not executed
		return (0);
	}
	
	//if door standing closed or in the midle
	if(!door.state && !SwEndOpen() ){
		
		if(door.restarted){
			
			//Set target speed for motor
			door.target_speed = speed;
		}
		else{

			//Set target speed for motor
			door.target_speed= DOOR_RESET_SPEED;
		}

		//Reset timer for target speed
		t_door_target_speed = 0;

		//Turn on motor to open door
		OutDoorOpen();
		
		//Set begin speed for motor
		DoorSpeedSet( DOOR_START_SPEED );
			
		//now motor running
		door.state = OPEN;		

	}
	//else if door opened and standing
	else if(SwEndOpen() && !door.state){
		
		//command is executed
		return (1);
	}
		
	//Run Application Service
	Service();

	//command is not executed
	return 0;
}

unsigned char StopDoor(void){

	//turn off motor only if motor running
	if( door.state ){
		
		//check if target speed is not Start speed
		if( DoorSpeedRead() != DOOR_START_SPEED){

			//set target speed to start speed
			door.target_speed = DOOR_START_SPEED;
		}
			
		//stop door easy and pick up counter events
		door.f_easy_stop = TRUE;
	
	}
	//if door standing
	else if( !door.state ){
		
		//command is executed
		return (1);
	}
	
	//Run Application Service
	Service();	

	//command is not executed
	return (0);
}

void LockDoor(void){
	
	if(lock_flag || lock_flag1 || lock_error)
		return;

	if( !SwDoorLocked()){
		
		//lock door only if door closed and is not locked
		if( SwEndClose() && !door.state ){
			
			//Turn on hardware PWM reduction 
			OutLockPwmOn();

			//turn on motor for lock door
			OutLockDoor();
			
			lock.state = LOCK;
			
			LockSpeedSet(160);

			t_lock = 0;

			lock_flag = TRUE;
		
		}	
	
	
	}

}

void UnlockDoor(void){

	if(lock_flag || lock_flag1 || lock_error )
		return;
	
	if( !SwDoorUnlocked() ){
		
		//Turn on hardware PWM reduction 
		OutLockPwmOn();

		//turn on motor for lock door
		OutUnLockDoor();

		lock.state = UNLOCK;			

		LockSpeedSet(160);

		t_lock = 0;

		lock_flag = TRUE;
	}
}

void LockDoor1(void){
	
	if(lock_flag || lock_flag1 || lock_error1)
		return;

	if( !SwDoorLocked1() && SwDoor1Closed() ){
		
		//Turn on hardware PWM reduction 
		OutLockPwmOn();

		//turn on motor for lock door
		OutLockDoor1();
			
		lock.state1 = LOCK;
			
		LockSpeedSet(160);

		t_lock = 0;

		lock_flag1 = TRUE;	
	
	}

}

void UnlockDoor1(void){

	if(lock_flag || lock_flag1 || lock_error1)
		return;

	if( !SwDoorUnlocked1() ){
		
		//Turn on hardware PWM reduction 
		OutLockPwmOn();

		//turn on motor for lock door
		OutUnLockDoor1();

		lock.state1 = UNLOCK;			

		LockSpeedSet(160);

		t_lock = 0;

		lock_flag1 = TRUE;
	}
}



void WaitApplication(unsigned char service_status, unsigned int time ){

	//reset application timer for waithing
	t_application = 0;

	//wait for application start ramp
	while(t_application < time){
		
		if( service_status ){
			
			Service();
		
		}
	}
}

void AppInit(void){
	
	SetPorts();
	
	StartTimer();
	
	StartPwm0();

	StartPwm2();
	
	EnableAdConverter();			  		//Enable voltage messuring, 10 bit's AD
	
	SetAdVoltageReference();		  		//Set max voltage value for max reg value
	
	EnableAdInterrupt();			  		//Enable AD complete interrupt
	
	SetAdDivFactor();				  		//Set prescaler for AD clock, XTAL/factor	
	
	SetMotorCounter();

	ts_block_again = 90;

	DDRE |= 2;

	sei();


}

void AppStart(void){

	//Set position out of range 
	door.position = 5440;

	WaitApplication(ENABLE, 2000);

	CloseDoor(255);
	
}

void AppRestart(void){
	
	//Set ports to default if external WDT fail
	SetPorts();
	
	//Stop PWM 0 if external WDT fail
	StopPwm0();
	
	//Stop PWM 2 if external WDT fail
	StopPwm2();

	PORTE |= 2;

	OutLedRunOff();
	OutLedErrorOn();	

	auto unsigned char pero;
	
	//forever loop
	while(1){
		
		for(pero = 0;pero<200;pero++){
			pero++;
			pero--;
		}

		OutStToggle();
		//Do nothing and wait external WDT restart
	}
}





