#include "config.h"
#include "board_drv.h"
#include "app.h"

static struct{
	unsigned char fontana;
	unsigned char saht;
	unsigned char wathering;
	unsigned char raining;
	unsigned char garden_light;
	unsigned char outside_light;
}sw;

static unsigned char wathering;
static unsigned char garden_light;
static unsigned char outside_light;
static unsigned char f_outside_light;

void GardenRun(void){
//___________________________Fontana Application_______________

	//check if fontana switch same like sw struct
	if( SwFontana() == sw.fontana){
		
		//reset timer for switch_fontana ramp
		t_sw_fontana = 0;
	}
	//wait switch_fontana ramp
	else if(t_sw_fontana > SW_RAMP){
		
		//update struct from Register
		sw.fontana = SwFontana();
		
		//reset timer for new switch_fontana ramp
		t_sw_fontana = 0;

		//Toggle fontana only on high edge
		if(sw.fontana){

			//toggle Fontana output
			OutFontanaToggle();
		}
	}
//___________________________Saht Application__________________
	
	//check if saht switch same like sw struct
	if( SwSaht() == sw.saht){
		
		//reset timer for switch_saht ramp
		t_sw_saht = 0;
	}
	//wait switch_saht ramp
	else if(t_sw_saht > SW_RAMP){
		
		//update struct from Register
		sw.saht = SwSaht();
		
		//reset timer for new switch_saht ramp
		t_sw_saht = 0;

		//Toggle saht only on high edge
		if(sw.saht){
			
			//toggle saht output
			OutSahtToggle();
		}
	}
//___________________________Wathering Application_____________
	
	//check if raining switch same like sw struct
	if( SwRaining() == sw.raining ){
		
		//reset timer for new switch_raining ramp
		t_sw_raining = 0;
	}
	//wait switch_raining ramp
	else if(t_sw_raining > SW_RAMP){

		//update struct from register
		sw.raining = SwRaining();

		//reset timer for new switch_raining ramp
		t_sw_raining = 0;
	}

	//check if wathering switch same like sw struct
	if( SwWathering() == sw.wathering ){
		
		//reset timer for new switch_wathering ramp
		t_sw_wathering = 0;
	}
	//wait switch_wathering ramp
	else if(t_sw_wathering > SW_RAMP){

		//update struct from register
		sw.wathering = SwWathering();
		
		//reset timer for new switch_wathering ramp
		t_sw_wathering = 0;

		//toggle wathering only on high edge
		if(sw.wathering){

			//if last wathering status was ENABLE
			if(wathering == ENABLE){
			
				//Turn off automatic wathering
				wathering = DISABLE;
				OutWatheringOff();
			}
			//else turn on automatic wathering
			else{
				wathering = ENABLE;
				OutWatheringOn();

				//reset timer for new automatic_wathering ramp
				ts_auto_wathering = 0;
			}		
		}
	}

	//if automatic wathering enabled
	if(wathering == ENABLE){
		
		//check rain sensor switch
		if( sw.raining ){
			
			//if garden wathering is on
			if(OutWatheringState() == ENABLE){

				//wait for automatic watheing ramp
				if(ts_auto_wathering > RAIN_TRIGER){
				
					//turn of wathering
					OutWatheringOff();
				}
			}
			else{
				//reset timer for new automatic_wathering ramp
				ts_auto_wathering = 0;
			}	
		}
		//if no rain present on sensor switch
		else if(!sw.raining){

			//if garden wathering is off
			if(OutWatheringState() == DISABLE){

				//wait for automatic_wathering ramp
				if(ts_auto_wathering > RAIN_TRIGER){
				
					//turn on wathering
					OutWatheringOn();
				}	
			}
			else{
				//reset timer for new automatic_wathering ramp
				ts_auto_wathering = 0;
			}
		}	
	}
//___________________________Garden Light Application__________
	
	//check if garden light switch same like sw struct
	if( SwGardenLight() == sw.garden_light){
		
		//reset timer for new switch_garden_light ramp
		t_sw_garden_light = 0;
	}
	//wait switch_garden_light ramp
	else if(t_sw_garden_light > SW_RAMP){
		
		//update struct from Register
		sw.garden_light = SwGardenLight();
		
		//reset timer for new switch_garden_light ramp
		t_sw_garden_light = 0;

		//Toggle garden ligght only on high edge
		if(sw.garden_light){
			
			//if last garde_light was ON
			if(garden_light == ENABLE){
			
				//Turn off garden light
				garden_light = DISABLE;
				OutGardenLightOff();
			}
			//else turn on garden light
			else{
				garden_light = ENABLE;
				OutGardenLightOn();

				//reset timer for new garden_light ramp
				t_garden_light = 0;
			}
		}
	}
	//if automatic garden light enabled
	if(garden_light == ENABLE){
		
		//if day (AD converter)
		if(ad_light < DAY_VOLTAGE ){
			
			//if garden light is on
			if(OutGardenLightState() == ENABLE){

				//wait for garden_light ramp
				if(t_garden_light > GARDEN_LIGHT_RAMP){
				
					//turn of garden light
					OutGardenLightOff();
				}
			}
			else{
				//reset timer for new garden light ramp
				t_garden_light = 0;
			}	
		}
		//if night (AD converter)
		else if(ad_light > NIGHT_VOLTAGE){

			//if garden light is off
			if(OutGardenLightState() == DISABLE){

				//wait for garden_light ramp 
				if(t_garden_light > GARDEN_LIGHT_RAMP){
					if(!block_low_voltage){
						//turn on garden light
						OutGardenLightOn();
					}
				}	
			}
			else{
				//reset timer for new garden_light ramp
				t_garden_light = 0;
			}
		}
		else{
			//reset timer for new garden_light ramp
			t_garden_light = 0;
		}	
	}
	if(block_low_voltage){

		OutGardenLightOff();

	}
//___________________________Outside Light Application_________
	
	//if day (AD converter)
	if(ad_light < DAY_VOLTAGE){
		
		//if outside_light status enabled
		if(outside_light == ENABLE){

			//wait outside_lite ramp
			if(t_outside_light > OUTSIDE_STATUS_RAMP ){

				//disable outside light because daylight present
				outside_light = DISABLE;
				
				//Turn off outside light	
				OutOutsideLightOff();

				//reset outside light execute flag
				f_outside_light = 0;
			}
		}
		else{

			//reset timer for new outside_light ramp
			t_outside_light = 0;
		}
	}
	//if night (AD convercet)
	else if(ad_light > NIGHT_VOLTAGE){
		
		//if outside light status disabled
		if(outside_light == DISABLE){
			
			//wait outside light ramp
			if(t_outside_light > OUTSIDE_STATUS_RAMP ){

				//enable outside light status 
				outside_light = ENABLE;
			}
		}
		else{
			
			//reset timer for new outside_light ramp
			t_outside_light = 0;
		}
	}
	else{
		
		//reset timer for new outside_light ramp
		t_outside_light = 0;
	}	


	//check if outside light switch same like sw struct
	if( SwOutsideLight() == sw.outside_light){
		
		//reset timer for outside_light ramp
		t_sw_outside_light = 0;
	}
	//wait outside_light ramp
	else if(t_sw_outside_light > SW_RAMP){
		
		//update struct from Register
		sw.outside_light = SwOutsideLight();
		
		//reset timer for outside_light ramp
		t_sw_outside_light = 0;
	}

	//if outside light enabled
	if( outside_light ){
		
		//if sw outside light active
		if( sw.outside_light ){
		
			//if flag for execute light is not activated
			if( !f_outside_light ){

				//set flag for execute outside light to 1
				f_outside_light = 1;

				//reset timer for execute outside light
				t_outside_light_exe = 0;
			}
			//else if execute status is 1
			else
			 if(f_outside_light == 1){
				
				//check for user key long presed
				if(t_outside_light_exe > 1000){

					//Turn on outside light
					OutOutsideLightOn();
			
					//Reset timer for execution
					t_outside_light_exe = 0;
			
					//Set flag for execute 1S turned on interval
					f_outside_light = 4;
				}
			}

			//disable overflow on execute timer
			if(t_outside_light_exe > 10000){
				
				//always pull to 10000 ms
				t_outside_light_exe = 10000;
			}
		}
		//if flag for execute is one
		else if( f_outside_light == 1 ){
		
			//if user key was presed < 1 S
			if(t_outside_light_exe <= 1000 ){
				
				//if real time bigger then 22 h
				if(After22()){

					//Execute second outside light solution
					f_outside_light = 2;
					
					//Reset timer for 3 S waithing
					t_outside_light_exe = 0;
				}
				else{

					//no valid execute, try protocol again
					f_outside_light = 0;
				}
			}
		}
		//if key is not presed and no switch sampling
		else if ( f_outside_light == 5 ){
			
			//reset execute flag for new execution	
			f_outside_light = 0;
		}

		//execute 3S waithing for execute light
		if(f_outside_light == 2){
			
			//if 3 S timeout
			if(t_outside_light_exe >= 3000){
				
				//Turn on outside light
				OutOutsideLightOn();
			
				//Reset timer for execution
				t_outside_light_exe = 0;
			
				//Set flag for execute 1S turned on interval
				f_outside_light = 4;
			}
		}

		//execute 1S light turned on interval
		if( f_outside_light == 4){
			
			//if timeout for active outside light
			if(t_outside_light_exe >= 1000){
				
				//turn off the light
				OutOutsideLightOff();
				
				//Set flag for restart protocol
				f_outside_light = 5;
			}
						
		}
		
	}
	
//_____________________________________________________________

}
