#include "app.h"
#include "haus_control.h"
#include "board_drv.h"
#include "config.h"

static unsigned char direction_timeout;

void MotorDriverRun(void){
//___________________________Door End Switches_________________
	
	//if door closed
	if( SwEndClose() ){
		
		door.restarted = TRUE;
		
		//stop door only if door closing
		if(door.state == CLOSE){
			
			door.f_stop = TRUE;
			
		}
		//else if door stoped
		else if( !door.state ){

			//reset begin position 
			door.position = 0;
		}

		//reset "try_close" for overcurrent function
		try_close = 0;
		try_open = 0;

	}

	//if door opened
	if( SwEndOpen() ){
		
		door.restarted = TRUE;
				
		//stop door only if door Opening
		if(door.state == OPEN){

			door.f_stop = TRUE;

		}
		//else if door stoped
		else if( !door.state ){

			//reset end position 
			door.position = 2720;
		}

		//reset "try_open" for overcurrent function
		try_open = 0;
		try_close = 0;
	}

//___________________________Stoping Motor_____________________

	if( door.f_stop == TRUE ){
		
		//check if pick up ramp actived
		if( door.f_pick_up == FALSE ){	
			
			//set motor speed to zero
			DoorSpeedSet(0);
		
			//Stop motor
			OutDoorStop();						

			//pick counter events 400 ms after motor off
			door.f_pick_up = TRUE;

			//reset timer for pick up counter events
			t_pick_counter_events = 0;
		
		}
	}
	//if easy stop flag actived
	if( door.f_easy_stop ){
		
		//if door speed = start speed
		if(DoorSpeedRead() == DOOR_START_SPEED){
			
			//Stop door and pick up events
			door.f_stop = TRUE;
		}
	}

//___________________________Pick Up Counter Events____________

	//if we need pick up counter events
	if( door.f_pick_up ){
			
		//if ramp for picking events timeout
		if(t_pick_counter_events > 200){	

			//reset event counter for new door running
			DoorCounterReset(0);

			//door is stoped
			door.state = FALSE;
			
			//del stop flag
			door.f_stop = FALSE;

			//reset pick up flag
			door.f_pick_up = FALSE;

			//reset easy stop flag
			door.f_easy_stop = 0;
			
		}
			
	}

	//if counter events bigger then zero
	if( DoorCounterRead()  ){
		
		//if door closing
		if(door.state == CLOSE){
			
			//door position = position - events pickuped
			door.position -= DoorCounterRead(); 
		}
		//else if door opening
		else if(door.state == OPEN){
			
			//door position = position + events pickuped
			door.position += DoorCounterRead();
		}

		//Delete counter events
		DoorCounterReset(0);
	}
//___________________________Motor Target speed________________

	//if door running check target speed 
	if( door.state && !door.f_pick_up ){
		
		//if speed lower then target speed
		if(DoorSpeedRead() < door.target_speed){
			
			//Chack timer for target speed
			if(t_door_target_speed >= INCR_SPEED_DELAY){
					
				//increment speed by one
				DoorSpeedIncr();

				//reset timer for new speed update
				t_door_target_speed = 0;
			}
		}
		//if speed higher then target speed
		else if(DoorSpeedRead() > door.target_speed){
			
			//Chack timer for target speed
			if(t_door_target_speed >= DECR_SPEED_DELAY){

				//decrement speed by one
				DoorSpeedDecr();

				//reset timer for new speed update
				t_door_target_speed = 0;
			}
		}
		else{
			//reset timer for new speed update
			t_door_target_speed = 0;
		
		}
	}

//___________________________Foto Ramp Service_________________

	//foto ramp is 400 ms for open door
	if(t_sw_foto < 400){
				
		//if sw foto relesed
		if( !SwFoto() ){
					
			//reset ramp for new sampling
			t_sw_foto = 0;
		}
	}

//___________________________Beep when door running____________		

#ifdef BEEP_STATUS

	//if door opening
	if(door.state == OPEN){

		//wait ramp 1S
		if(t_beep >= 1000){
			
			//reset beep ramp for new ramp waithing
			t_beep = 0;

			//toggle buzz output
			OutBeepToggle();			

		}
		
	}
	//else if dorr closing
	else if(door.state == CLOSE){
		
		//if buzz activated
		if( OutBeepState() ){

			//wait ramp short beep
			if(t_beep >= 350){
				
				//reset ramp for new ramp waithing
				t_beep = 0;

				//turn off buzzer
				OutBeepOff();			
			}
		}
		//else if buzz deactived
		else{
			
			//wait long ramp
			if(t_beep >= 650){
				
				//reset ramp for new ramp waithing
				t_beep = 0;

				//turn on buzzer
				OutBeepOn();			

			}
		}
	}
	//if motor stoped and buzz turned on 
	else if( OutBeepState() ){
		
		//Turn off buzzer
		OutBeepOff();
	}
#endif

	//if door running
	if( door.state ){

		//if flash light turned off
		if( !OutLightFlashState() ){
			
			//Turn on Flash light
			OutLightFlashOn();
		}		
	
	}
	//else if light flashing 
	else if( OutLightFlashState() ){
			
			//Turn off Flash light
			OutLightFlashOff();
	
	}
//___________________________Crytical Door Sections____________
	
	//if door state "close" & position smaller then 100
	if((door.position <= 100) && (door.state == CLOSE)){
		
		//Slow down motor to crash speed
		door.target_speed = DOOR_CRASH_SPEED;
	}
	//if door staate "open" & position bigger then 2620
	else if((door.position >= 2620) && (door.state == OPEN)){
		
		//Slow down motor to crash speed
		door.target_speed = DOOR_CRASH_SPEED;
	}

//___________________________Users Keys driver_________________
	
	//if switch state same like struct variable
	if(SwUserOpen() == door.sw_user_open){
		
		//reset ramp for new sampling
		t_sw_user_open = 0;
		
		if( t_when_sw_open > 600 ){

			door.sw_user_open_lng = TRUE;
		}		
	}
	else if(t_sw_user_open > SW_RAMP){
		
		//update struct variable with new value
		door.sw_user_open = SwUserOpen();
		
		//reset ramp for new sampling
		t_sw_user_open = 0; 
		
		//remambar time when switch change state
		if( door.sw_user_open ){

			door.sw_user_open_lng = FALSE;

			t_when_sw_open = 0;
		}
		
	}
	//if switch state same like struct variable
	if(SwUserClose() == door.sw_user_close){
		
		//reset ramp for new sampling
		t_sw_user_close = 0;
		
		if((t_when_sw_close > 600) && door.sw_user_close){

			door.sw_user_close_lng = TRUE;
		}
	}
	else if(t_sw_user_close > SW_RAMP){
		
		//update struct variable with new value
		door.sw_user_close = SwUserClose();
		
		//reset ramp for new sampling
		t_sw_user_close = 0;
		
		//remambar time when switch  presed
		if( door.sw_user_close ){
			
			door.sw_user_close_lng = FALSE;
				
			t_when_sw_close = 0; 
		}
		
	}
//___________________________Switch user lock small door_______
	//if switch state same like struct variable
	if(SwUserLock1() == lock.sw_user_lock1){
		
		//reset ramp for new sampling
		t_sw_user_lock1 = 0;

		if( t_when_sw_lock1 > 100 ){

			lock.sw_user_lock1_lng = TRUE;
		}
	}
	else if(t_sw_user_lock1 > SW_RAMP){
		
		//update struct variable with new value
		lock.sw_user_lock1 = SwUserLock1();
		
		//reset ramp for new sampling
		t_sw_user_lock1 = 0; 
		
		//remambar time when switch change state
		if( lock.sw_user_lock1 ){

			lock.sw_user_lock1_lng = FALSE;

			t_when_sw_lock1 = 0;
		}
		
	}
	//if switch state same like struct variable
	if(SwUserUnlock1() == lock.sw_user_unlock1){
		
		//reset ramp for new sampling
		t_sw_user_unlock1 = 0;

		if( t_when_sw_unlock1 > 100 ){

			lock.sw_user_unlock1_lng = TRUE;
		}
	}
	else if(t_sw_user_unlock1 > SW_RAMP){
		
		//update struct variable with new value
		lock.sw_user_unlock1 = SwUserUnlock1();
		
		//reset ramp for new sampling
		t_sw_user_unlock1 = 0; 
		
		//remambar time when switch change state
		if( lock.sw_user_unlock1 ){

			lock.sw_user_unlock1_lng = FALSE;

			t_when_sw_unlock1 = 0;
		}
		
	}
	
//___________________________OverCurrent Motor Service_________
	
	//if 4A current is same like in variable
	if( SwOverCurrent4A() == over_current_4A){
		
		//reset 4A current timer for new sampling
		t_over_current_4A = 0;
	}

	else if(t_over_current_4A >= CURRENT_RAMP) {
		
		//Update current variable with new value
		over_current_4A = SwOverCurrent4A();
		
		//reset 4A current timer for new sampling
		t_over_current_4A = 0;
	}

	//if 8A current is same like in variable
	if( SwOverCurrent8A() == over_current_8A){
		
		//reset 8A current timer for new sampling
		t_over_current_8A = 0;
	}

	else if(t_over_current_8A >= CURRENT_RAMP) {
		
		//Update current variable with new value
		over_current_8A = SwOverCurrent8A();
		
		//reset 8A current timer for new sampling
		t_over_current_8A = 0;

	}
//___________________________Auto Close Door Service___________
	
	//if door runing 
	if( door.state ){
		
		//Reset timer for auto closing door
		ts_auto_close = 0;
	}

//___________________________Timeout for one direction_________
	
	//if door running
	if(door.state){
		
		if( !door.restarted ){
			//reset timer for new direction
			ts_direction_timeout = 0;
		}
		//if direction changed
		if(direction_timeout != door.state){
			
			//set new direction for timeout
			direction_timeout = door.state;
			
			//reset timer for new direction
			ts_direction_timeout = 0;
		}
		//if timeout for direction 30 S
		else if(ts_direction_timeout > 30){
			
			//Restart procesor
			AppRestart();
		
		}
	
	}
	//id door standing
	else{

		//Reset direction timer for new direction ramp
		ts_direction_timeout = 0;
	}

//___________________________Foto Sensor Tempreture____________
	if( !block_low_voltage ){	
		//if temperature > or = High voltage
		if(ad_foto_temp >= H_FOTO_VOLTAGE){
		
			//if foto temperature out turned off
			if( OutFotoState()){
			
				//Turn on out foto temp
				OutFotoOff();
			}
		}
		//if temperature < or = High voltage
		else if(ad_foto_temp <= L_FOTO_VOLTAGE){
		
			//If foto temperature out turned on
			if(!OutFotoState()){
		
				//Turn off out foto temp
				OutFotoOn();
			}
		}
		else if((ad_foto_temp <= 460) && (SwRaining() == 1)){
			
			//If foto temperature out turned on
			if(!OutFotoState()){
		
				//Turn off out foto temp
				OutFotoOn();
			}
		}
	}
	else if(OutFotoState()){
		
		//Turn on out foto temp
		OutFotoOff();
	}

//___________________________High temperature service__________

	if(ad_high_temp > 549){
		
		block_pwm = TRUE;
	}
	else if(ad_high_temp < 492){
	
		block_pwm = FALSE;
	}
//___________________________Block User Key Service____________
	
	//check if fontana switch same like sw struct
	if( SwitchBlock() == user_block_key){
		
		//reset timer for switch_fontana ramp
		t_user_block_key = 0;
	}
	//wait switch_fontana ramp
	else if(t_user_block_key > SW_RAMP){
		
		//update struct from Register
		user_block_key = SwitchBlock();
		
		//reset timer for new switch_fontana ramp
		t_user_block_key = 0;
	}

//___________________________Low Woltage Service_______________
	if(ad_low_voltage < LOW_VOLTAGE_L){

		block_low_voltage = TRUE;
	}
	else if(ad_low_voltage > LOW_VOLTAGE_H){

		block_low_voltage = FALSE;
	}
//___________________________End Of Motor Driver_______________
//___________________________End Of Motor Driver_______________
}
